#!/bin/sh
# Talsyntes r30debug
case "$-" in *i*) return ;; esac
export LC_ALL=C
#exec 2>&-
REVISION=$(sed -n '2s/^\s*#\s*\(Talsyntes\s*\)\?//p' "$0")
APP="Talsyntes $REVISION"
DIR=~/".talsyntes"
SCRIPT="$DIR/talsyntes.sh"
DATA_DIR="$DIR/data"
VOICES_DIR="$DIR/voices"
SETTINGS_DIR="$DIR/settings"
BINDINGS_DIR="$DIR/bindings"

mkdir -p "$DATA_DIR" "$VOICES_DIR" "$SETTINGS_DIR" "$BINDINGS_DIR"

if test "x$0" != "x$SCRIPT"
then
   grep -q "^$(sed -n '2p' "$0")$" "$SCRIPT" || set install
   if cp -f "$0" "$SCRIPT" 2>&-
   then
      iconv -f utf-8 -t iso8859-1 "$0" >/dev/null 2>&- ||
      iconv -f iso8859-1 -t utf-8 "$0" >"$SCRIPT"
      chmod +x "$SCRIPT" && exec "$SCRIPT" "$@"
   fi
fi
get_pids() {
   ps -opid=,ppid=,args= $(pgrep -u"$(id -u)" ${3:+-P}$3 "$1") |
   sed -n "/^\s*\([0-9]\+\s\+\)\?$$\s\+/d;s|^\(\s*[0-9]\+\).*$2|\1:|;s/:.*//p"
}
kill_pids() {
   while read pid
   do pkill "$@" -P "$pid" ; kill "$@" "$pid"; false
   done 2>&-
}
is_open() {
   get_pids "${SCRIPT##*/}" " $SCRIPT $*$" | grep -q .
}
close() {
   if get_pids "${SCRIPT##*/}" " $SCRIPT $*$" | kill_pids
   then false
   else
      case "$1" in
      import) dialog="$APP --edit" ;;
      *) false ;;
      esac &&
      get_pids '(zenity|Xdialog)' "$dialog" | kill_pids ;:
   fi
}
case "$1" in
pause)
   find "$DIR" -name mplayer.\*.fifo -type p -exec sh -c 'echo pause >{}' \;
;;
stop|start|reset|prev|next|restart|loop|lang)
   pgrep -u"$(id -u)" -P $(pgrep -u"$(id -u)" -d, "${SCRIPT##*/}") mplayer |
   while read pid
   do kill $(ps -oppid= "$pid") "$pid"
   done
;;
timeout)
   close "$*" && exit
   sleep 0.5 && shift
   exec "$SCRIPT" "$@" & exit
;;
install)
   close "$*"
;;
*) is_open "$*" && exit ;;
esac

create_dialog() {
   dialog="$(get_dialog)"
   case "$1" in
   list)
      type="$2"
      title="$3"
      while read item
      do echo "$item" | grep -q "\($4\)" && default=true || default=false
         case "$dialog" in
         *zenity)
            set "$@" "$default" "${item##*/}"
         ;;
         *Xdialog)
            set "$@" "${item##*/}" "${item##*/}"
            "$default" && set "$@" on || set "$@" off
         ;;
         esac
      done
      shift 4
      case "$dialog" in
      *zenity)
         set -- --width 400 --height 500 --list \
            --"$type"list --text "$title" --column "" --column "" "$@"
      ;;
      *Xdialog)
         set -- --stdout --separator "|" --no-tags \
            --"$type"list "$title" 400x500 0 "$@"
      ;;
      esac
   ;;
   scale)
      case "$dialog" in
      *zenity)
         set -- --scale --text "$2" --print-partial \
            --min-value -100 --max-value 100 --value "$3"
      ;;
      *Xdialog)
         set -- --stdout --rangebox "$2" 10 60 -100 100 "$3"
      ;;
      esac
   ;;
   progress)
echo "\$dialog:[$dialog]"
      case "$dialog" in
      *zenity)
         set -- --width 300 --progress --auto-close --text "$2"
      ;;
      *Xdialog)
         set -- --stdout --gauge "$2" 5 60
      ;;
      esac
   ;;
   edit)
      case "$dialog" in
      *zenity)
         set -- --editable --text-info \
            --filename "${2:-/dev/null}" --width 300 --height 500
      ;;
      *Xdialog)
         set -- --editbox "${2:-/dev/null}" 30 50
      ;;
      esac
   ;;
   yesno)
      case "$dialog" in
      *zenity)
         set -- --question --no-wrap --text "$2"
      ;;
      *kdialog)
         set -- --yesno "$2"
      ;;
      *Xdialog)
         set -- --fixed-font --yesno "$2" 0 0 0
      ;;
      esac
   ;;
   error)
      case "$dialog" in
      *zenity)
         set -- --error --no-wrap --text "$2"
      ;;
      *kdialog)
         set -- --error "$2"
      ;;
      *Xdialog)
         set -- --fixed-font --infobox "$2" 0 0 0
      ;;
      esac
   ;;
   msgbox)
      case "$dialog" in
      *zenity)
         set -- --info --no-wrap --text "$2"
      ;;
      *kdialog)
         set -- --msgbox "$2"
      ;;
      *Xdialog)
         set -- --fixed-font --msgbox "$2" 0 0 0
      ;;
      esac
   ;;
   *) return ;;
   esac
   #GTK2_RC_FILES="$DIR"/gtkrc \
   LC_ALL=sv_SE.UTF-8 "${dialog:-:}" --title "$APP" "$@" 2>&-
}
scale_mapping() {
   awk -vmin="$1" -vdef="$2" -vmax="$3" -vbend="${4:-1}" '
      function f(x)
      {
         factor = (a != b) ? 2*a*b*log(a/b)/(a-b) : (a+b)
         return factor * log(x) * x / (x^2-1) - ((a < b) ? a : b) / bend
      }
      BEGIN {
         a = (max - def)
         b = (def - min)
         if (max < min) { a = -a; b = -b; }
         x0=2; x1=3; y0=f(x0); y1=f(x1); lim=1e-4
         while( x1>1 && ( (x1-x0)>lim || (x0-x1)>lim) ) {
            y1 = f(x1); x2 = x1 - (x1 - x0) * y1 / (y1 - y0)
            x0 = x1; x1 = x2; y0 = y1;
         }
         incl = (x1>1) ? x1 : 1
      }
      {
         expo = $1 / 100
         if (incl != 1) expo = (incl^expo - 1/incl^expo) / (incl - 1/incl)
         expo = (expo + 1) / 2
         if (a != b) expo = ((a / b)^(2 * expo) - 1) / ((a / b)^2 - 1)
         printf("%.3f\n", min + (max - min) * expo)
      }
   '
   return 0
}

get_amplit() {
   2>&- scale_mapping 0 100 200 <"$SETTINGS_DIR"/amplit || echo 100
}
get_wordgap() {
   2>&- scale_mapping -50 0 50 <"$SETTINGS_DIR"/wordgap || echo 0
}
get_pitch() {
   2>&- scale_mapping 0 50 99 <"$SETTINGS_DIR"/pitch || echo 50
}
get_espeed() {
   2>&- scale_mapping 80 170 370 <"$SETTINGS_DIR"/espeed || echo 170
}
get_speed() {
   2>&- scale_mapping 5 1 0.2 <"$SETTINGS_DIR"/speed || echo 1.0
}
get_freq() {
   2>&- scale_mapping 0.2 1 5 <"$SETTINGS_DIR"/freq || echo 1.0
}
get_volume() {
   2>&- scale_mapping 0.2 1 5 <"$SETTINGS_DIR"/volume || echo 1.0
}
get_rate() {
   2>&- scale_mapping 1000 16000 48000 <"$SETTINGS_DIR"/rate || echo 16000
}
get_scale() {
   2>&- scale_mapping 0.2 1 5 <"$SETTINGS_DIR"/scale || echo 1.0
}
get_stride() {
   2>&- scale_mapping 1 60 400 <"$SETTINGS_DIR"/stride || echo 60
}
get_overlap() {
   2>&- scale_mapping 0 0.2 1 <"$SETTINGS_DIR"/overlap || echo 0.2
}
get_search() {
   2>&- scale_mapping 0 14 200 <"$SETTINGS_DIR"/search || echo 14
}
get_method() {
   2>&- cat "$SETTINGS_DIR"/method || echo tempo
}
get_accel() {
   2>&- scale_mapping 0.2 1 5 <"$SETTINGS_DIR"/accel || echo 1.0
}
get_voice() {
   ls "$VOICES_DIR/$voice":* 2>&- ||
   readlink "$SETTINGS_DIR"/voice 2>&- ||
   ls "$VOICES_DIR"/[a-z][a-z][1-9]:* | head -n1
}
get_dialog() {
   which "$dialog" 2>&- ||
   which $(cat "$SETTINGS_DIR"/dialog 2>&-) 2>&- ||
   which zenity 2>&- || which Xdialog 2>&- || which kdialog 2>&- ||
   echo "$0: zenity eller Xdialog saknas" >&2
}
get_hand() {
   2>&- cat "$SETTINGS_DIR"/hand || echo right
}

find -L "$SETTINGS_DIR" -maxdepth 1 -name voice -type l -exec rm {} \;
ls "$VOICES_DIR"/[a-z][a-z][1-9]:* >/dev/null 2>&- &&
test -x "$SCRIPT" &&
test -x "$DIR/mbrola" ||
set install

cd "$DIR"
case "$1" in
stop|pause)
;;
prev)
   line=$(cat "$DIR"/line 2>&-)
   expr "${line:-1}" - 1 >"$DIR"/line
   exec "$SCRIPT" start
;;
next)
   line=$(cat "$DIR"/line 2>&-)
   expr "${line:-1}" + 1 >"$DIR"/line
   exec "$SCRIPT" start
;;
restart)
   echo 1 >"$DIR"/line
   exec "$SCRIPT" start
;;
loop)
   while "$SCRIPT" restart ; do :; done; exit
;;
lang)
   voice="$(get_voice)"
   voice=$(
      case "${voice%:*}" in
      (*/en[1-9]|*/us[1-9])
         ls "$VOICES_DIR/sw"[1-9]:*"${voice##* }" ||
         ls "$VOICES_DIR/sw"[1-9]:*
      ;;
      (*-en)
         ls "${voice%-en:*}":*
      ;;
      (*)
         ls "${voice%:*}-en":* ||
         ls "$VOICES_DIR/en"[1-9]:*"${voice##* }" ||
         ls "$VOICES_DIR/us"[1-9]:*"${voice##* }" ||
         ls "$VOICES_DIR/en"[1-9]:* ||
         ls "$VOICES_DIR/us"[1-9]:*
      ;;
      esac ||
      ls "$VOICES_DIR"/[a-z][a-z][1-9]:*
   )
   voice=$(echo "$voice" | head -n1)
   voice="${voice##*/}"
   voice="${voice%:*}" exec "$SCRIPT" start
;;
start)
   CLIP=$(which xclip 2>&- || which xsel 2>&-) &&
   SPEAK=$(which espeak 2>&-) &&
   PLAYER=$(which mplayer 2>&-) ||
   exec "$SCRIPT" install
   echo ${SCALETEMPO:-true} >"$SETTINGS_DIR"/scaletempo
   find "$DIR" -name mplayer.\*.fifo -type p -exec rm -f {} \;
   FIFO="$DIR/mplayer.$$.fifo"
   TMP="$(mktemp)"
   trap 'rm -f "$FIFO" "$TMP"' EXIT
   test -p "$FIFO" || mkfifo "$FIFO"
   voice="$(get_voice)"
   voice="${voice##*/}"
   : ${amplit="$(get_amplit)"}
   : ${wordgap="$(get_wordgap)"}
   : ${pitch="$(get_pitch)"}
   : ${espeed="$(get_espeed)"}
   : ${speed="$(get_speed)"}
   : ${freq="$(get_freq)"}
   : ${volume="$(get_volume)"}
   : ${rate="$(get_rate)"}
   : ${accel="$(get_accel)"}

   scaletempo="-speed ${accel:-1}"
   if $SCALETEMPO
   then
      : ${scale="$(get_scale)"}
      : ${stride="$(get_stride)"}
      : ${overlap="$(get_overlap)"}
      : ${search="$(get_search)"}
      : ${method="$(get_method)"}

      scaletempo="$scaletempo -af scaletempo="
      scaletempo="$scaletempo${scaletempo+:}scale=${scale:-1}"
      scaletempo="$scaletempo${scaletempo+:}stride=${stride:-60}"
      scaletempo="$scaletempo${scaletempo+:}overlap=${overlap:-0.20}"
      scaletempo="$scaletempo${scaletempo+:}search=${search:-14}"
      scaletempo="$scaletempo${scaletempo+:}speed=${method:-tempo}"
   fi

#   freq=$(awk '{print(0.953-(exp($1/19.3)-1)/278.66)}' <"$SETTINGS_DIR"/speed)
#   rate=$(awk '{print(17229+(exp($1/19.3)-1)*100)}' <"$SETTINGS_DIR"/speed)

   ${CLIP:-:} -o |
   awk -F'[ \t\n]*[?.,!][ \t\n]' '{
      gsub("\\u[0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f][0-9A-Fa-f]","")
      gsub("[ \t](and|or)[ \t\n]","\n&")
      for(i=1;i<=NF;i++) {
         printf(" %s ", $i)
         if(i==NF || match($(i+1),"[ \t]")) {
            printf("\n")
         }
      }
   }' >"$TMP"
   line=$(cat "$DIR"/line 2>&-)
   test -s "$TMP" &&
   if cmp -s "$TMP" "$DIR"/text
   then :
   else mv "$TMP" "$DIR"/text && line=1
   fi
   test "${line:-1}" -gt $(2>&- wc -l <"$DIR"/text) 2>&- ||
   test "${line:-1}" -lt 1 && line=1
   sed -n "${line:-1},\$p" "$DIR"/text 2>&- |
   while read sentence
   do echo "[$$] ${line:-1}) $sentence"
      echo "$sentence" |
      ${SPEAK:-:} \
         -a "${amplit:-100}" \
         -g "${wordgap:-0}" \
         -p "${pitch:-50}" \
         -s "${espeed:-170}" \
         -v mb-"${voice%:*}" |
      "$DIR/mbrola" -e \
         -t "${speed:-1}" \
         -f "${freq:-1}" \
         -v "${volume:-1}" \
         -l "${rate:-16000}" \
         "$VOICES_DIR/$voice" - -.wav |
      ${PLAYER:-:} \
         -really-quiet -input file="$FIFO" $scaletempo -demuxer audio - ||
      exec env SCALETEMPO=false "$SCRIPT" start
      line=$(expr "${line:-1}" + 1)
      echo "$line" >"$DIR"/line
   done
;;
about)
   about=$(
      echo "$APP"
      echo
      echo "Startad:" $(stat -c%y "$SCRIPT")
   )
   create_dialog msgbox "$about"
;;
help)
   help=$(
      sed -n "
         \|^\"$SCRIPT help\"$|{
            n;s/$/:\tLista över tangentkommandon/p}
         \|^\"$SCRIPT pause\"$|{
            n;s/$/:\tPausa uppläsningen/p}
         \|^\"$SCRIPT start\"$|{
            n;s/$/:\tStarta om nuvarande mening/p}
         \|^\"$SCRIPT prev\"$|{
            n;s/$/:\tStarta om föregående mening/p}
         \|^\"$SCRIPT next\"$|{
            n;s/$/:\tHoppa till nästa mening/p}
         \|^\"$SCRIPT voice\"$|{
            n;s/$/:\tVälj röst och språk/p}
         \|^\"$SCRIPT speed\"$|{
            n;s/$/:\tJustera uppläsningsfarten/p}
         \|^\"$SCRIPT lang\"$|{
            n;s/$/:\tNuvarande mening på annat språk/p}
         \|^\"speed=1\s*$SCRIPT start\"$|{
            n;s/$/:\tNuvarande mening i normal fart/p}
         \|^\"speed=0.75\s*$SCRIPT start\"$|{
            n;s/$/:\tNuvarande mening i snabb fart/p}
         \|^\"speed=0.5\s*$SCRIPT start\"$|{
            n;s/$/:\tNuvarande mening i mycket snabb fart/p}
         \|^\"$SCRIPT install\"$|{
            n;s/$/:\tInstallera fler röster/p}
         \|^\"$SCRIPT stop\"$|{
            n;s/$/:\tStoppa uppläsningen/p}
         \|^\"$SCRIPT restart\"$|{
            n;s/$/:\tStarta om hela markeringen/p}
         \|^\"$SCRIPT speed -[0-9]\+\"$|{
            n;s/$/:\tMinska uppläsningsfarten/p}
         \|^\"$SCRIPT speed +[0-9]\+\"$|{
            n;s/$/:\tÖka uppläsningsfarten/p}
         \|^\"$SCRIPT adjust\"$|{
            n;s/$/:\tJustera röstparametrar/p}
         \|^\"$SCRIPT combo\"$|{
            n;s/$/:\tJustera röstparametrergrupper/p}
         \|^\"$SCRIPT rebind[^\"]*\"$|{
            n;s/$/:\tByt tangentbindingsläge/p}
         \|^\"$SCRIPT accel\"$|{
            n;s/$/:\tJustera röstparameter 'accel'/p}
         \|^\"$SCRIPT volume\"$|{
            n;s/$/:\tJustera uppläsningsvolymen/p}
         \|^\"$SCRIPT scale\"$|{
            n;s/$/:\tJustera röstparameter 'scale'/p}
         \|^\"$SCRIPT stride\"$|{
            n;s/$/:\tJustera röstparameter 'stride'/p}
         \|^\"$SCRIPT search\"$|{
            n;s/$/:\tJustera röstparameter 'search'/p}
         \|^\"$SCRIPT overlap\"$|{
            n;s/$/:\tJustera röstparameter 'overlap'/p}
         \|^\"$SCRIPT reset\"$|{
            n;s/$/:\tNollställ alla röstparametrar/p}
         \|^\"$SCRIPT loop\"$|{
            n;s/$/:\tRepitera nuvarande mening/p}
         \|^\"$SCRIPT dialog\"$|{
            n;s/$/:\tÄndra dialogrutesystem/p}
         \|^\"$SCRIPT method\"$|{
            n;s/$/:\tJustera röstparameter 'method'/p}
         \|^\"$SCRIPT import\"$|{
            n;s/$/:\tImportera röstparameterlista/p}
         \|^\"$SCRIPT espeed\"$|{
            n;s/$/:\tJustera röstparameter 'espeed'/p}
         \|^\"$SCRIPT amplit\"$|{
            n;s/$/:\tJustera röstparameter 'amplit'/p}
         \|^\"$SCRIPT freq\"$|{
            n;s/$/:\tJustera röstläget/p}
         \|^\"$SCRIPT rate\"$|{
            n;s/$/:\tJustera rösttätheten/p}
         \|^\"$SCRIPT pitch\"$|{
            n;s/$/:\tJustera röstparameter 'pitch'/p}
         \|^\"$SCRIPT wordgap\"$|{
            n;s/$/:\tJustera röstparameter 'wordgap'/p}
         \|^\"$SCRIPT espeak\"$|{
            n;s/$/:\tJustera röstparametrar 'espeak'/p}
         \|^\"$SCRIPT mbrola\"$|{
            n;s/$/:\tJustera röstparametrar 'mbrola'/p}
         \|^\"$SCRIPT mplayer\"$|{
            n;s/$/:\tJustera röstparametrar 'mplayer'/p}
         \|^\"$SCRIPT scaletempo\"$|{
            n;s/$/:\tJustera röstparametrar 'scaletempo'/p}
         \|^\"$SCRIPT about\"$|{
            n;s/$/:\tVersionsinformation/p}
         \|^\"$SCRIPT quit\"$|{
            n;s/$/:\tAvsluta $APP/p}
         \|^\"$SCRIPT uninstall\"$|{
            n;s/$/:\tAvinstallera $APP/p}
      " <"$DIR"/keys
   )
   $(cat "$SETTINGS_DIR"/scaletempo 2>&-) ||
   help=$(echo "$help" | grep -vE '(scale|stride|overlap|search|method)')
   keys=$(readlink "$DIR"/keys)
   help=$(
      echo "Tangentkommandon: ${keys##*/}"
      echo
      echo "$help" | sed -n 's/^F[0-9]\+:/&\t\t/p' | sort -tF -nk2
      echo "$help" | sed -n '/^Shift\s\++\s\+F[0-9]\+:/p' | sort -tF -nk2
      echo "$help" | sed -n 's/^Alt\s\++\s\+F[0-9]:/&\t/p' | sort -tF -nk2
      echo "$help" | sed -n '/^Alt\s\++\s\+F[0-9][0-9]:/p' | sort -tF -nk2
      echo "$help" | grep -vE '^(F[0-9]+:|Shift|Alt)[[:blank:]]' | sort
   )
   create_dialog msgbox "$help"
;;
import)
   close import
   cd "$SETTINGS_DIR"
   TMP="$(mktemp)"
   trap 'rm -f "$TMP"' EXIT
   case "$2" in
   '')
      OPTIONS="amplit wordgap pitch espeed speed freq rate volume accel"
      $(cat "$SETTINGS_DIR"/scaletempo 2>&-) &&
      OPTIONS="$OPTIONS scale stride overlap search method"
      echo "Nuvarande inställningar"
      echo
      for option in voice $OPTIONS dialog
      do if test -h "$option"
         then
            setting=$(readlink "$option")
            setting="${setting##*/}" && value=""
         else
            setting="$(cat $option 2>&-)"
            value="$(get_$option)"
         fi
         test "$option" = dialog &&
         setting="${value##*/}" && value=""
         test "$option" = method &&
         setting="${setting:-tempo}" && value=""
         printf "%s:\t\t%s" "$option" "${setting:-0}"
         test -n "$value" && printf "\t\t(%s)\n" "$value" || echo
      done #| sort
   ;;
   empty) ;;
   *) cat <"$2" 2>&- ;;
   esac >"$TMP"
   create_dialog edit "$TMP" |
   while read option setting real_value
   do echo $option,$setting,$real_value
      case "${option%:}" in
      voice|dialog|method)
         "$SCRIPT" adjust voice "${setting%:}"
      ;;
      scale|stride|overlap|search|method| \
      amplit|wordgap|pitch|espeed|speed|freq|rate|volume|accel)
         echo "$setting" |
         awk '{x=0+$1;if(x<-100)x=-100;if(x>100)x=100;print(x)}' >"${option%:}"
      ;;
      esac
   done
;;
reset)
   cd "$SETTINGS_DIR"
   case "$2" in
   '')
      create_dialog yesno "Återställ standardinställningarna?" || exit
      rm -f amplit wordgap pitch espeed dialog speed freq volume \
         rate scale stride overlap search method accel scaletempo
   ;;
   amplit|wordgap|pitch|espeed|dialog|speed|freq|volume|rate|accel|scaletempo)
      rm -f "$2"
   ;;
   scale|stride|overlap|search|method)
      $(cat "$SETTINGS_DIR"/scaletempo 2>&-) && rm -f "$2"
   ;;
   esac
   pkill -9 "${SCRIPT##*/}"
;;
adjust)
   shift
   case "$2" in
   espeak|mbrola|mplayer|scaletempo| \
   combo|voice|dialog|hand|scale|stride|overlap|search|method| \
   amplit|wordgap|pitch|espeed|speed|freq|volume|rate|accel)
      "$SCRIPT" adjust "$1" & shift ; exec "$SCRIPT" adjust "$@"
   ;;
   *) if test -n "$3"
      then "$SCRIPT" adjust "$1" "$2" & shift 2 ; exec "$SCRIPT" adjust "$@"
      fi
   ;;
   esac
   case "$1" in
   scaletempo|scale|stride|overlap|search|method)
      $(cat "$SETTINGS_DIR"/scaletempo 2>&-) || exit ;;
   esac
   value=$(cat "$SETTINGS_DIR/$1" 2>&-)
   case "$1" in
   '')
      OPTIONS="amplit wordgap pitch espeed speed freq volume rate accel"
      $(cat "$SETTINGS_DIR"/scaletempo 2>&-) &&
      OPTIONS="$OPTIONS scale stride overlap search method"
      set -- $(
         printf "%s\n" voice $OPTIONS hand dialog |
         create_dialog list radio "Välj inställning" '^speed$'
      )
      test -n "$1" && exec "$SCRIPT" adjust "$@"
   ;;
   combo)
      OPTIONS="espeak mbrola mplayer"
      $(cat "$SETTINGS_DIR"/scaletempo 2>&-) &&
      OPTIONS="$OPTIONS scaletempo"
      set -- $(
         printf "%s\n" $OPTIONS |
         create_dialog list radio "Välj inställningsgrupp" '^mbrola$'
      )
      test -n "$1" && exec "$SCRIPT" adjust "$@"
   ;;
   espeak) exec "$SCRIPT" adjust espeed pitch wordgap amplit ;;
   mbrola) exec "$SCRIPT" adjust speed freq rate volume ;;
   mplayer) exec "$SCRIPT" adjust accel ;;
   scaletempo) exec "$SCRIPT" adjust scale stride overlap search method ;;
   voice)
      case "$2" in
      '')
         voice=$(readlink "$SETTINGS_DIR/$1")
         voice=$(
            ls "$VOICES_DIR"/[a-z][a-z][1-9][:-]* 2>&- |
            create_dialog list radio "Välj röst" "^${voice#../}$"
         ) && test -n "$voice" && voice="$VOICES_DIR/$voice"
      ;;
      *) voice=$(ls "$VOICES_DIR/$2":* 2>&-) ;;
      esac && ln -sf "$voice" "$SETTINGS_DIR/$1"
   ;;
   dialog)
      case "$2" in
      '')
         value=$(
            which zenity Xdialog 2>&- |
            create_dialog list radio "Välj dialogsystem" "/${value:-zenity}$"
         ) && test -n "$value" && echo "$value" >"$SETTINGS_DIR/$1"
      ;;
      zenity|Xdialog)
         echo "$2" >"$SETTINGS_DIR/$1" ;;
      esac
   ;;
   hand)
      case "$2" in
      '')
         value=$(
            printf "%s\n" right left 2>&- |
            create_dialog list radio "Välj mushand" "^${value:-right}$"
         ) && test -n "$value" && echo "$value" >"$SETTINGS_DIR/$1"
      ;;
      right|left)
         echo "$2" >"$SETTINGS_DIR/$1" ;;
      esac
      case $(readlink "$DIR"/keys) in
      */left|*/right) exec "$SCRIPT" rebind ;;
      esac
   ;;
   method)
      case "$2" in
      '')
         value=$(
            printf "%s\n" tempo pitch both none |
            create_dialog list radio "scaletempo: $1" "^${value:-tempo}$"
         ) && test -n "$value" && echo "$value" >"$SETTINGS_DIR/$1"
      ;;
      tempo|pitch|both|none)
         echo "$2" >"$SETTINGS_DIR/$1" ;;
      esac
   ;;
   *) case "$1" in
      amplit|wordgap|pitch|espeed) title="espeak: $1" ;;
      speed) title="Uppläsningshastighet" ;;
      freq) title="Röstläge" ;;
      volume) title="Uppläsningsvolym" ;;
      rate) title="Rösttäthet" ;;
      accel) title="mplayer: $1" ;;
      scale|stride|overlap|search) title="scaletempo: $1" ;;
      *) false ;;
      esac &&
      case "$2" in
      '')
         { create_dialog scale "$title" "${value:-0}" || echo "$value"; } |
         while read value
         do test -n "$value" && echo "$value" >"$SETTINGS_DIR/$1"
         done
      ;;
      *) echo "$2" |
         awk -vvalue="$value" '{
            value = ($1) ? value + $1 : 0
            value = (value < 100) ? value : 100
            value = (value > -100) ? value : -100
            print(value)
         }' >"$SETTINGS_DIR/$1"
        exec "$SCRIPT" start
      ;;
      esac
   ;;
   esac
;;
unbind)
   close help
   ps -opid=,args= $(pgrep -u"$(id -u)" -d, xbindkeys) 2>&- |
   sed -n "s|\s\+xbindkeys -f $DIR/.*||p" |
   while read pid
   do kill -9 "$pid"
   done
;;
rebind)
   "$SCRIPT" unbind
   ln -sf "${BINDINGS_DIR#$DIR/}/${2:-$(get_hand)}" "$DIR"/keys
   setsid xbindkeys -f "$DIR"/keys
   exec "$SCRIPT" help
;;
install)
   #echo 'gtk-font-name = "Sans 20"' >"$DIR"/gtkrc
   dep=$(
      test -f /lib/ld-linux.so.2 || echo libc6-i386
      for dep in 'Xdialog zenity' xbindkeys espeak 'xsel xclip' mplayer
      do for dep in $dep
         do which "$dep" >/dev/null 2>&- && break
         done || echo "$dep"
      done
   )
echo "\$dep:[$dep]"
   if test -n "$dep"
   then
      SUDO="$(which gksudo 2>&- || which kdesudo 2>&- || which kdesu 2>&-)"
      which apt-get >/dev/null 2>&- || SUDO=""
      case "$SUDO" in
      */gksudo) ;;
      */kdesudo|*/kdesu) SUDO="$SUDO --noignorebutton" ;;
      *) SUDO="" ; DIALOG_TYPE=error ;;
      esac
echo "\$SUDO:[$SUDO]"
      create_dialog "${DIALOG_TYPE:-yesno}" \
         "Följande saknas:\n\n$dep${SUDO:+\n\nInstallera?}" &&
      ${SUDO:-false} -- 'apt-get -q -y install' $dep &&
      exec "$SCRIPT" install || exit
   fi
echo A
   URL="http://tcts.fpms.ac.be/synthesis/mbrola"
   TMP="$(mktemp)" && test -n "${TMP%%/*}" && TMP="$(pwd)/$TMP"
   trap 'rm -f "$TMP"' EXIT
echo B
   wget -qO- "$URL"/mbrcopybin.html |
   sed -n '
      s|.*"\(bin/[^"]*.zip\)"[^>]*>[^<]*LINUX i386.*|\1 (0.3Mb)|p
      s|.*"\(dba/[^/]*/[^"]*\)"[^>]*>\([^:]*:\s*[^<]*\).*|\1\t\2|p
   ' >"$TMP"
echo C
   voices=$(
      cd /usr/share/espeak-data/voices/mb &&
      echo mb-[a-z][a-z][1-9] |
      sed 's/mb-//g;s/\(.\)\s\(.\)/\1|\2/g'
   )
echo D
   voices=$(
      grep -E "/($voices)/" <"$TMP" | cut -f2 |
      create_dialog list check "Välj språk" '^\(sw\|us\|en\)[[:digit:]]:' |
      sed 's/:[^|]*//g'
   )
echo "\$voices:[$voices]"
   test -n "$voices" || exit
   rm -f "$VOICES_DIR"/[a-z][a-z][1-9][:-]* 2>&-
   cd "$DATA_DIR"
echo E
   grep -E "(^bin|/$voices/)" <"$TMP" |
   awk '{
      size[NR]=size[NR-1]+substr($NF,2,length($NF)-4)
      $NF=""
      voice[NR]=$0
   } END {
      for(i=1; i<=NR; i++) {
         print(100*size[i]/size[NR],voice[i])
      }
   }' |
   while read progress file voice
   do
echo "\$progress:[$progress]"
echo "\$file:[$file]"
echo "\$voice:[$voice]"
      test -f "${file##*/}" ||
      wget -q "$URL/$file" 2>&1
      if test -z "$voice"
      then
         unzip -p "${file##*/}" mbrola-linux-i386 >"$DIR"/mbrola
         chmod +x "$DIR"/mbrola
      else
         unzip -p "${file##*/}" "${voice%%:*}" >"$VOICES_DIR/$voice" 2>&- ||
         unzip -p "${file##*/}" "${voice%%:*}/${voice%%:*}" \
            >"$VOICES_DIR/$voice"
         test -e /usr/share/espeak-data/voices/mb/mb-"${voice%%:*}"-en &&
         ln -s "$voice" "$VOICES_DIR/${voice%:*}-en:${voice#*:} (English)"
      fi
      echo $progress
   done |
   create_dialog progress "Hämtar Röster..."
echo F
   printf '"%s"\n%s\n\n' \
                   "$SCRIPT help"            "F1" \
                   "$SCRIPT rebind adjust"   "Shift + F1" \
      "speed=1      $SCRIPT start"           "F2" \
      "speed=0.75   $SCRIPT start"           "F3" \
      "speed=0.5    $SCRIPT start"           "F4" \
                   "$SCRIPT lang"            "F5" \
                   "$SCRIPT scaletempo"      "Shift + F2" \
                   "$SCRIPT mplayer"         "Shift + F3" \
                   "$SCRIPT espeak"          "Shift + F4" \
                   "$SCRIPT mbrola"          "Shift + F5" \
                   "$SCRIPT speed"           "F6" \
                   "$SCRIPT adjust"          "Shift + F6" \
                   "$SCRIPT voice"           "F7" \
                   "$SCRIPT install"         "Shift + F7" \
                   "$SCRIPT next"            "F8" \
                   "$SCRIPT prev"            "F9" \
                   "$SCRIPT speed +10"       "Shift + F8" \
                   "$SCRIPT speed -10"       "Shift + F9" \
                   "$SCRIPT start"           "F10" \
                   "$SCRIPT restart"         "Shift + F10" \
                   "$SCRIPT pause"           "F11" \
                   "$SCRIPT stop"            "Shift + F11" \
                   "$SCRIPT about"           "F12" \
                   "$SCRIPT quit"            "Shift + F12" \
   >"$BINDINGS_DIR"/left
   printf '"%s"\n%s\n\n' \
                   "$SCRIPT help"            "F1" \
                   "$SCRIPT rebind adjust"   "Shift + F1" \
                   "$SCRIPT pause"           "F2" \
                   "$SCRIPT stop"            "Shift + F2" \
                   "$SCRIPT start"           "F3" \
                   "$SCRIPT restart"         "Shift + F3" \
                   "$SCRIPT prev"            "F4" \
                   "$SCRIPT next"            "F5" \
                   "$SCRIPT speed -10"       "Shift + F4" \
                   "$SCRIPT speed +10"       "Shift + F5" \
                   "$SCRIPT speed"           "F6" \
                   "$SCRIPT adjust"          "Shift + F6" \
                   "$SCRIPT voice"           "F7" \
                   "$SCRIPT install"         "Shift + F7" \
                   "$SCRIPT mbrola"          "Shift + F8" \
                   "$SCRIPT espeak"          "Shift + F9" \
                   "$SCRIPT mplayer"         "Shift + F10" \
                   "$SCRIPT scaletempo"      "Shift + F11" \
                   "$SCRIPT lang"            "F8" \
      "speed=1      $SCRIPT start"           "F9" \
      "speed=0.75   $SCRIPT start"           "F10" \
      "speed=0.5    $SCRIPT start"           "F11" \
                   "$SCRIPT about"           "F12" \
                   "$SCRIPT quit"            "Shift + F12" \
   >"$BINDINGS_DIR"/right
   printf '"%s"\n%s\n\n' \
                   "$SCRIPT help"            "F1" \
                   "$SCRIPT rebind"          "Shift + F1" \
                   "$SCRIPT stop"            "F2" \
                   "$SCRIPT reset"           "Shift + F2" \
                   "$SCRIPT restart"         "F3" \
                   "$SCRIPT loop"            "Shift + F3" \
                   "$SCRIPT import"          "F4" \
                   "$SCRIPT combo"           "F5" \
                   "$SCRIPT adjust"          "F6" \
                   "$SCRIPT voice"           "F7" \
                   "$SCRIPT method"          "Shift + F4" \
                   "$SCRIPT search"          "Shift + F5" \
                   "$SCRIPT overlap"         "Shift + F6" \
                   "$SCRIPT stride"          "Shift + F7" \
                   "$SCRIPT scale"           "Shift + F8" \
                   "$SCRIPT accel"           "F8" \
                   "$SCRIPT speed"           "F9" \
                   "$SCRIPT espeed"          "Shift + F9" \
                   "$SCRIPT freq"            "F10" \
                   "$SCRIPT pitch"           "Shift + F10" \
                   "$SCRIPT rate"            "F11" \
                   "$SCRIPT wordgap"         "Shift + F11" \
                   "$SCRIPT volume"          "F12" \
                   "$SCRIPT amplit"          "Shift + F12" \
   >"$BINDINGS_DIR"/adjust
   #test -s ~/.xsession || echo >>~/.xsession
   #grep -q "^xbindkeys -f \"$DIR/" ~/.xsession ||
   #sed -i "1ixbindkeys -f \"$DIR/keys\"" ~/.xsession
   "$SCRIPT" voice
   "$SCRIPT" rebind
;;
uninstall)
   create_dialog yesno "Avinstallera ${APP% $REVISION}?" || exit
   #sed -i "\|^xbindkeys -f \"$DIR/|d" ~/.xsession
   rm -rf "$DIR"
   create_dialog msgbox "${APP% $REVISION} avinstallerad"
;;
quit)
   get_pids "${SCRIPT##*/}" " $SCRIPT" | kill_pids
   get_pids 'zenity' 'zenity' | kill_pids
   get_pids 'Xdialog' 'Xdialog' | kill_pids
   exec "$SCRIPT" unbind
;;
pack)
   tar cjf "${SCRIPT%.sh}-$REVISION.tar.bz2" "${SCRIPT##*/}"
;;
voice|dialog|hand|combo|espeak|mbrola|mplayer| \
amplit|wordgap|pitch|espeed|speed|freq|volume|rate|accel)
   exec "$SCRIPT" adjust "$@"
;;
scaletempo|scale|stride|overlap|search|method)
   $(cat "$SETTINGS_DIR"/scaletempo 2>&-) &&
   exec "$SCRIPT" adjust "$@"
;;
'')
   exec "$SCRIPT" rebind "$@"
;;
*)
   exec "$SCRIPT" start "$@"
;;
esac
